@@ -18,6 +18,9 @@ from litdata.streaming.combined import CombinedStreamingDataset
 from litdata.streaming.dataloader import StreamingDataLoader
 from litdata.streaming.dataset import StreamingDataset
 from litdata.streaming.item_loader import TokensLoader
+from litdata.streaming.writer import index_parquet_dataset
+from litdata.utilities.breakpoint import breakpoint
+from litdata.utilities.hf_dataset import index_hf_dataset
 from litdata.utilities.train_test_split import train_test_split
 
 __all__ = [
@@ -30,6 +33,9 @@ __all__ = [
     "walk",
     "train_test_split",
     "merge_datasets",
+    "index_parquet_dataset",
+    "index_hf_dataset",
+    "breakpoint",
 ]
 if RequirementCache("lightning_sdk"):
     from lightning_sdk import Machine  # noqa: F401
