@@ -22,13 +22,15 @@ from datetime import datetime
 from functools import partial
 from pathlib import Path
 from types import FunctionType
-from typing import Any, Callable, Dict, List, Literal, Optional, Sequence, Tuple, Union
+from typing import TYPE_CHECKING, Any, Callable, Dict, List, Literal, Optional, Sequence, Tuple, Union
 from urllib import parse
 
 import torch
 
-from litdata.constants import _INDEX_FILENAME, _IS_IN_STUDIO
-from litdata.processing.data_processor import DataChunkRecipe, DataProcessor, DataTransformRecipe
+from litdata import __version__
+from litdata.constants import _INDEX_FILENAME, _IS_IN_STUDIO, _SUPPORTED_PROVIDERS
+from litdata.helpers import _check_version_and_prompt_upgrade
+from litdata.processing.data_processor import DataChunkRecipe, DataProcessor, MapRecipe
 from litdata.processing.readers import BaseReader
 from litdata.processing.utilities import (
     _get_work_dir,
@@ -36,8 +38,8 @@ from litdata.processing.utilities import (
     optimize_dns_context,
     read_index_file_content,
 )
-from litdata.streaming.client import S3Client
 from litdata.streaming.dataloader import StreamingDataLoader
+from litdata.streaming.fs_provider import _get_fs_provider
 from litdata.streaming.item_loader import BaseItemLoader
 from litdata.streaming.resolver import (
     Dir,
@@ -50,10 +52,13 @@ from litdata.utilities._pytree import tree_flatten
 from litdata.utilities.encryption import Encryption
 from litdata.utilities.format import _get_tqdm_iterator_if_available
 
+if TYPE_CHECKING:
+    from lightning_sdk import Machine
+
 
 def _is_remote_file(path: str) -> bool:
     obj = parse.urlparse(path)
-    return obj.scheme in ["s3", "gcs"]
+    return obj.scheme in _SUPPORTED_PROVIDERS
 
 
 def _get_indexed_paths(data: Any) -> Dict[int, str]:
@@ -98,9 +103,16 @@ def _get_default_num_workers() -> int:
     return os.cpu_count() or 1
 
 
-class LambdaDataTransformRecipe(DataTransformRecipe):
-    def __init__(self, fn: Callable[[str, Any], None], inputs: Union[Sequence[Any], StreamingDataLoader]):
-        super().__init__()
+class LambdaMapRecipe(MapRecipe):
+    """Recipe for `map`."""
+
+    def __init__(
+        self,
+        fn: Callable[[str, Any], None],
+        inputs: Union[Sequence[Any], StreamingDataLoader],
+        storage_options: Dict[str, Any] = {},
+    ):
+        super().__init__(storage_options)
         self._fn = fn
         self._inputs = inputs
         self._device: Optional[str] = None
@@ -142,6 +154,8 @@ class LambdaDataTransformRecipe(DataTransformRecipe):
 
 
 class LambdaDataChunkRecipe(DataChunkRecipe):
+    """Recipe for `optimize`."""
+
     def __init__(
         self,
         fn: Callable[[Any], None],
@@ -151,8 +165,15 @@ class LambdaDataChunkRecipe(DataChunkRecipe):
         compression: Optional[str],
         encryption: Optional[Encryption] = None,
         existing_index: Optional[Dict[str, Any]] = None,
+        storage_options: Dict[str, Any] = {},
     ):
-        super().__init__(chunk_size=chunk_size, chunk_bytes=chunk_bytes, compression=compression, encryption=encryption)
+        super().__init__(
+            chunk_size=chunk_size,
+            chunk_bytes=chunk_bytes,
+            compression=compression,
+            encryption=encryption,
+            storage_options=storage_options,
+        )
         self._fn = fn
         self._inputs = inputs
         self.is_generator = False
@@ -180,7 +201,7 @@ class LambdaDataChunkRecipe(DataChunkRecipe):
         return self._inputs
 
     def prepare_item(self, item_metadata: Any) -> Any:
-        """This method is overriden dynamically."""
+        """Being overridden dynamically."""
 
 
 def map(
@@ -192,17 +213,20 @@ def map(
     num_workers: Optional[int] = None,
     fast_dev_run: Union[bool, int] = False,
     num_nodes: Optional[int] = None,
-    machine: Optional[str] = None,
+    machine: Optional[Union["Machine", str]] = None,
     num_downloaders: Optional[int] = None,
     num_uploaders: Optional[int] = None,
     reorder_files: bool = True,
     error_when_not_empty: bool = False,
     reader: Optional[BaseReader] = None,
     batch_size: Optional[int] = None,
+    start_method: Optional[str] = None,
+    optimize_dns: Optional[bool] = None,
+    storage_options: Dict[str, Any] = {},
 ) -> None:
-    """This function maps a callable over a collection of inputs, possibly in a distributed way.
+    """Maps a callable over a collection of inputs, possibly in a distributed way.
 
-    Arguments:
+    Args:
         fn: A function to be executed over each input element
         inputs: A sequence of input to be processed by the `fn` function, or a streaming dataloader.
         output_dir: The folder where the processed data should be stored.
@@ -218,9 +242,15 @@ def map(
         reorder_files: By default, reorders the files by file size to distribute work equally among all workers.
             Set this to ``False`` if the order in which samples are processed should be preserved.
         error_when_not_empty: Whether we should error if the output folder isn't empty.
+        reader: The reader to use when reading the data. By default, it uses the `BaseReader`.
         batch_size: Group the inputs into batches of batch_size length.
-
+        start_method: The start method used by python multiprocessing package. Default to spawn unless running
+            inside an interactive shell like Ipython.
+        optimize_dns: Whether the optimized dns should be used.
+        storage_options: Storage options for the cloud provider.
     """
+    _check_version_and_prompt_upgrade(__version__)
+
     if isinstance(inputs, StreamingDataLoader) and batch_size is not None:
         raise ValueError("When providing a streaming dataloader, pass the batch_size to the dataloader directly.")
 
@@ -253,11 +283,11 @@ def map(
         if _output_dir.url and "cloudspaces" in _output_dir.url:
             raise ValueError(
                 f"The provided `output_dir` isn't valid. Found {_output_dir.path if _output_dir else None}."
-                " HINT: You can either use `/teamspace/s3_connections/...` or `/teamspace/datasets/...`."
+                "\n HINT: You can either use `/teamspace/s3_connections/...` or `/teamspace/datasets/...`."
             )
 
         if error_when_not_empty:
-            _assert_dir_is_empty(_output_dir)
+            _assert_dir_is_empty(_output_dir, storage_options=storage_options)
 
         if not isinstance(inputs, StreamingDataLoader):
             input_dir = input_dir or _get_input_dir(inputs)
@@ -281,9 +311,12 @@ def map(
             reorder_files=reorder_files,
             weights=weights,
             reader=reader,
+            start_method=start_method,
+            storage_options=storage_options,
         )
-        with optimize_dns_context(True):
-            return data_processor.run(LambdaDataTransformRecipe(fn, inputs))
+
+        with optimize_dns_context(optimize_dns if optimize_dns is not None else False):
+            return data_processor.run(LambdaMapRecipe(fn, inputs, storage_options=storage_options))
     return _execute(
         f"litdata-map-{datetime.now().strftime('%Y-%m-%d_%H-%M-%S')}",
         num_nodes,
@@ -291,6 +324,24 @@ def map(
     )
 
 
+#
+# `Optimize` Pipeline:
+#
+# 1. optimize() function uses LambdaDataChunkRecipe to process inputs
+# 2. LambdaDataChunkRecipe is passed to DataProcess
+# 3. DataProcess spawns DataWorkerProcess (BaseWorker) instances
+# 4. Each worker:
+#    a. Processes a single input through the optimize fn
+#    b. Flattens the output using pytree:
+#       - Converts nested structures (dict, list, tuple) into flat list
+#       - Generates data_spec to preserve structure for later reconstruction
+#    c. Uses Writer to serialize (bytes) the flattened data
+#    d. Writes chunk files when either limit is reached:
+#       - Total bytes > chunk_bytes
+#       - Total items > chunk_size
+#    e. Creates chunk_{idx}.bin files in cache
+#    f. Saves data_spec in index.json for data structure preservation
+#
 def optimize(
     fn: Callable[[Any], Any],
     inputs: Union[Sequence[Any], StreamingDataLoader],
@@ -304,7 +355,7 @@ def optimize(
     num_workers: Optional[int] = None,
     fast_dev_run: bool = False,
     num_nodes: Optional[int] = None,
-    machine: Optional[str] = None,
+    machine: Optional[Union["Machine", str]] = None,
     num_downloaders: Optional[int] = None,
     num_uploaders: Optional[int] = None,
     reorder_files: bool = True,
@@ -314,10 +365,12 @@ def optimize(
     use_checkpoint: bool = False,
     item_loader: Optional[BaseItemLoader] = None,
     start_method: Optional[str] = None,
+    optimize_dns: Optional[bool] = None,
+    storage_options: Dict[str, Any] = {},
 ) -> None:
     """This function converts a dataset into chunks, possibly in a distributed way.
 
-    Arguments:
+    Args:
         fn: A function to be executed over each input element. The function should return the data sample that
             corresponds to the input. Every invocation of the function should return a similar hierarchy of objects,
             where the object types and list sizes don't change.
@@ -336,6 +389,7 @@ def optimize(
         machine: When doing remote execution, the machine to use. Only supported on https://lightning.ai/.
         num_downloaders: The number of downloaders per worker.
         num_uploaders: The numbers of uploaders per worker.
+        reader: The reader to use when reading the data. By default, it uses the `BaseReader`.
         reorder_files: By default, reorders the files by file size to distribute work equally among all workers.
             Set this to ``False`` if the order in which samples are processed should be preserved.
         batch_size: Group the inputs into batches of batch_size length.
@@ -347,8 +401,11 @@ def optimize(
                 the format in which the data is stored and optimized for loading.
         start_method: The start method used by python multiprocessing package. Default to spawn unless running
             inside an interactive shell like Ipython.
-
+        optimize_dns: Whether the optimized dns should be used.
+        storage_options: Storage options for the cloud provider.
     """
+    _check_version_and_prompt_upgrade(__version__)
+
     if mode is not None and mode not in ["append", "overwrite"]:
         raise ValueError(f"The provided `mode` should be either `append` or `overwrite`. Found {mode}.")
 
@@ -370,8 +427,7 @@ def optimize(
 
     if not _IS_IN_STUDIO and (machine is not None or num_nodes is not None):
         raise ValueError(
-            "Only https://lightning.ai/ supports multiple nodes or selecting a machine."
-            "Create an account to try it out."
+            "Only https://lightning.ai/ supports multiple nodes or selecting a machine.Create an account to try it out."
         )
 
     if not _IS_IN_STUDIO:
@@ -398,10 +454,12 @@ def optimize(
         if _output_dir.url is not None and "cloudspaces" in _output_dir.url:
             raise ValueError(
                 f"The provided `output_dir` isn't valid. Found {_output_dir.path}."
-                " HINT: You can either use `/teamspace/s3_connections/...` or `/teamspace/datasets/...`."
+                "\n HINT: You can either use `/teamspace/s3_connections/...` or `/teamspace/datasets/...`."
             )
 
-        _assert_dir_has_index_file(_output_dir, mode=mode, use_checkpoint=use_checkpoint)
+        _assert_dir_has_index_file(
+            _output_dir, mode=mode, use_checkpoint=use_checkpoint, storage_options=storage_options
+        )
 
         if not isinstance(inputs, StreamingDataLoader):
             resolved_dir = _resolve_dir(input_dir or _get_input_dir(inputs))
@@ -417,7 +475,9 @@ def optimize(
         num_workers = num_workers or _get_default_num_workers()
         state_dict = {rank: 0 for rank in range(num_workers)}
 
-        existing_index_file_content = read_index_file_content(_output_dir) if mode == "append" else None
+        existing_index_file_content = (
+            read_index_file_content(_output_dir, storage_options) if mode == "append" else None
+        )
 
         if existing_index_file_content is not None:
             for chunk in existing_index_file_content["chunks"]:
@@ -439,9 +499,10 @@ def optimize(
             use_checkpoint=use_checkpoint,
             item_loader=item_loader,
             start_method=start_method,
+            storage_options=storage_options,
         )
 
-        with optimize_dns_context(True):
+        with optimize_dns_context(optimize_dns if optimize_dns is not None else False):
             data_processor.run(
                 LambdaDataChunkRecipe(
                     fn,
@@ -451,6 +512,7 @@ def optimize(
                     compression=compression,
                     encryption=encryption,
                     existing_index=existing_index_file_content,
+                    storage_options=storage_options,
                 )
             )
         return None
@@ -468,7 +530,7 @@ def _listdir(folder: str) -> Tuple[str, List[str]]:
 class walk:
     """This class is an optimized version of os.walk for listing files and folders from cloud filesystem.
 
-    Note: The order of files and folders yielded aren't depth-first anymore due to the asynchronous listing call.
+    .. note:: The order of files and folders yielded aren't depth-first anymore due to the asynchronous listing call.
 
     """
 
@@ -519,14 +581,19 @@ class CopyInfo:
     new_filename: str
 
 
-def merge_datasets(input_dirs: List[str], output_dir: str) -> None:
-    """The merge_datasets utility enables to merge multiple existing optimized datasets into a single optimized
-    dataset.
+def merge_datasets(
+    input_dirs: List[str],
+    output_dir: str,
+    max_workers: Optional[int] = os.cpu_count(),
+    storage_options: Dict[str, Any] = {},
+) -> None:
+    """Enables to merge multiple existing optimized datasets into a single optimized dataset.
 
-    Arguments:
+    Args:
         input_dirs: A list of directories pointing to the existing optimized datasets.
         output_dir: The directory where the merged dataset would be stored.
-
+        max_workers: Number of workers for multithreading
+        storage_options: Storage options for the cloud provider.
     """
     if len(input_dirs) == 0:
         raise ValueError("The input directories needs to be defined.")
@@ -536,16 +603,17 @@ def merge_datasets(input_dirs: List[str], output_dir: str) -> None:
 
     resolved_input_dirs = [_resolve_dir(input_dir) for input_dir in input_dirs]
     resolved_output_dir = _resolve_dir(output_dir)
+    max_workers = max_workers or 1
 
     if any(input_dir == resolved_output_dir for input_dir in resolved_input_dirs):
         raise ValueError("The provided output_dir was found within the input_dirs. This isn't supported.")
 
-    input_dirs_file_content = [read_index_file_content(input_dir) for input_dir in resolved_input_dirs]
+    input_dirs_file_content = [read_index_file_content(input_dir, storage_options) for input_dir in resolved_input_dirs]
 
     if any(file_content is None for file_content in input_dirs_file_content):
         raise ValueError("One of the provided input_dir doesn't have an index file.")
 
-    output_dir_file_content = read_index_file_content(resolved_output_dir)
+    output_dir_file_content = read_index_file_content(resolved_output_dir, storage_options)
 
     if output_dir_file_content is not None:
         raise ValueError("The output_dir already contains an optimized dataset")
@@ -579,13 +647,19 @@ def merge_datasets(input_dirs: List[str], output_dir: str) -> None:
 
     _tqdm = _get_tqdm_iterator_if_available()
 
-    for copy_info in _tqdm(copy_infos):
-        _apply_copy(copy_info, resolved_output_dir)
+    with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
+        futures: List[concurrent.futures.Future] = []
+        for copy_info in copy_infos:
+            future = executor.submit(_apply_copy, copy_info, resolved_output_dir, storage_options)
+            futures.append(future)
 
-    _save_index(index_json, resolved_output_dir)
+        for future in _tqdm(concurrent.futures.as_completed(futures), total=len(futures)):
+            future.result()
+
+    _save_index(index_json, resolved_output_dir, storage_options)
 
 
-def _apply_copy(copy_info: CopyInfo, output_dir: Dir) -> None:
+def _apply_copy(copy_info: CopyInfo, output_dir: Dir, storage_options: Dict[str, Any] = {}) -> None:
     if output_dir.url is None and copy_info.input_dir.url is None:
         assert copy_info.input_dir.path
         assert output_dir.path
@@ -595,20 +669,16 @@ def _apply_copy(copy_info: CopyInfo, output_dir: Dir) -> None:
         shutil.copyfile(input_filepath, output_filepath)
 
     elif output_dir.url and copy_info.input_dir.url:
-        input_obj = parse.urlparse(os.path.join(copy_info.input_dir.url, copy_info.old_filename))
-        output_obj = parse.urlparse(os.path.join(output_dir.url, copy_info.new_filename))
+        input_filepath = os.path.join(copy_info.input_dir.url, copy_info.old_filename)
+        output_filepath = os.path.join(output_dir.url, copy_info.new_filename)
 
-        s3 = S3Client()
-        s3.client.copy(
-            {"Bucket": input_obj.netloc, "Key": input_obj.path.lstrip("/")},
-            output_obj.netloc,
-            output_obj.path.lstrip("/"),
-        )
+        fs_provider = _get_fs_provider(output_dir.url, storage_options)
+        fs_provider.copy(input_filepath, output_filepath)
     else:
         raise NotImplementedError
 
 
-def _save_index(index_json: Dict, output_dir: Dir) -> None:
+def _save_index(index_json: Dict, output_dir: Dir, storage_options: Dict[str, Any] = {}) -> None:
     if output_dir.url is None:
         assert output_dir.path
         with open(os.path.join(output_dir.path, _INDEX_FILENAME), "w") as f:
@@ -619,11 +689,7 @@ def _save_index(index_json: Dict, output_dir: Dir) -> None:
 
             f.flush()
 
-            obj = parse.urlparse(os.path.join(output_dir.url, _INDEX_FILENAME))
+            remote_path = os.path.join(output_dir.url, _INDEX_FILENAME)
 
-            s3 = S3Client()
-            s3.client.upload_file(
-                f.name,
-                obj.netloc,
-                obj.path.lstrip("/"),
-            )
+            fs_provider = _get_fs_provider(output_dir.url, storage_options)
+            fs_provider.upload_file(f.name, remote_path)
