@@ -25,10 +25,10 @@ __SAMPLES_KEY__ = "__SAMPLES__"
 
 
 class CombinedStreamingDataset(IterableDataset):
-    """The `CombinedStreamingDataset` enables to stream data from multiple StreamingDataset with the sampling ratio of
+    """Enables to stream data from multiple StreamingDataset with the sampling ratio of
     your choice.
 
-    Addtionally, the `CombinedStreamingDataset` keeps track of the number of samples fetched to enable resumability
+    Additionally, the `CombinedStreamingDataset` keeps track of the number of samples fetched to enable reusability
     of the datasets.
 
     Note that due to the random sampling, the number of samples returned from the iterator is variable and a function
@@ -42,22 +42,25 @@ class CombinedStreamingDataset(IterableDataset):
         seed: int = 42,
         weights: Optional[Sequence[float]] = None,
         iterate_over_all: bool = True,
+        force_override_state_dict: bool = False,
     ) -> None:
-        """ "
-        Arguments:
+        """Enable to stream data from multiple StreamingDataset with the sampling ratio of your choice.
+
+        Args:
             datasets: The list of the StreamingDataset to use.
             seed: The random seed to initialize the sampler
             weights: The sampling ratio for the datasets
             iterate_over_all: When iterate_over_all is True, the combined dataset iterates over all the datasets.
                 Otherwise, it stops as soon as one raises a StopIteration.
+            force_override_state_dict: Boolean flag for allowing local arguments to override a loaded state dict.
         """
-
         self._check_datasets(datasets)
 
         self._seed = seed
         self._datasets = datasets
         self._weights = weights
         self._iterate_over_all = iterate_over_all
+        self._force_override_state_dict = force_override_state_dict
 
         if iterate_over_all and weights:
             raise ValueError(
@@ -181,13 +184,21 @@ class CombinedStreamingDataset(IterableDataset):
             return
 
         if len(state_dict["dataset"]) != len(self._datasets):
-            raise RuntimeError(f"The provided state doesn't match the current number of datasets: {self._datasets}.")
+            if not self._force_override_state_dict:
+                raise RuntimeError(
+                    f"The provided state doesn't match the current number of datasets: {self._datasets}."
+                )
+            if len(state_dict["dataset"]) > len(self._datasets):
+                raise RuntimeError(
+                    "Currently it's only possible to add datasets to the end of the dataset list when overriding state"
+                )
 
         for dataset_idx, dataset in enumerate(self._datasets):
-            if str(dataset_idx) not in state_dict["dataset"]:
-                raise RuntimeError(f"The provided state doesn't contain the index {dataset_idx}.")
+            if str(dataset_idx) in state_dict["dataset"]:
+                dataset.load_state_dict(state_dict["dataset"][str(dataset_idx)])
 
-            dataset.load_state_dict(state_dict["dataset"][str(dataset_idx)])
+            elif not self._force_override_state_dict:
+                raise RuntimeError(f"The provided state doesn't contain the index {dataset_idx}.")
 
         # Used to iterate over the sampler to avoid sampling the same samples
         if self._use_streaming_dataloader:
@@ -210,7 +221,7 @@ class _CombinedDatasetIterator(Iterator):
         self._num_samples_yielded = num_samples_yielded or [0 for _ in range(len(datasets))]
         self._original_weights = deepcopy(weights)
         self._weights = deepcopy(weights)
-        self._rng = random.Random(seed)
+        self._rng = random.Random(seed)  # noqa: S311
         self._iterate_over_all = iterate_over_all
         self._is_done = False
 
