@@ -12,11 +12,12 @@
 # limitations under the License.
 
 import os
+from collections import defaultdict
 from typing import Any, Dict, List, Optional, Tuple
 
 from litdata.constants import _INDEX_FILENAME
 from litdata.streaming.compression import _COMPRESSORS, Compressor
-from litdata.streaming.downloader import get_downloader_cls
+from litdata.streaming.downloader import get_downloader
 from litdata.streaming.item_loader import BaseItemLoader, Interval, PyTreeLoader, TokensLoader
 from litdata.streaming.sampler import ChunkedIndex
 from litdata.streaming.serializers import Serializer
@@ -35,14 +36,14 @@ class ChunksConfig:
         region_of_interest: Optional[List[Tuple[int, int]]] = None,
         storage_options: Optional[Dict] = {},
     ) -> None:
-        """The ChunksConfig reads the index files associated a chunked dataset and enables to map an index to its
-        chunk.
+        """Reads the index files associated a chunked dataset and enables to map an index to its chunk.
 
         Arguments:
             cache_dir: The path to cache folder.
             serializers: The serializers used to serialize and deserialize the chunks.
             remote_dir: The path to a remote folder where the data are located.
                 The scheme needs to be added to the path.
+            item_loader: The item loader used to load the data from the chunks.
             subsampled_files: List of subsampled chunk files loaded from `input_dir/index.json` file.
             region_of_interest: List of tuples of {start,end} of region of interest for each chunk.
             storage_options: Additional connection options for accessing storage services.
@@ -69,7 +70,8 @@ class ChunksConfig:
         else:
             self._chunks = load_subsampled_chunks(subsampled_files, _original_chunks)
 
-        self._config["data_spec"] = treespec_loads(self._config["data_spec"])
+        if self._config["data_spec"] is not None:
+            self._config["data_spec"] = treespec_loads(self._config["data_spec"])
 
         assert self._chunks is not None
         self._item_loader.setup(self._config, self._chunks, serializers, region_of_interest)
@@ -78,7 +80,7 @@ class ChunksConfig:
         self._downloader = None
 
         if remote_dir:
-            self._downloader = get_downloader_cls(remote_dir, cache_dir, self._chunks, self._storage_options)
+            self._downloader = get_downloader(remote_dir, cache_dir, self._chunks, self._storage_options)
 
         self._compressor_name = self._config["compression"]
         self._compressor: Optional[Compressor] = None
@@ -96,6 +98,11 @@ class ChunksConfig:
 
         self._skip_chunk_indexes_deletion: Optional[List[int]] = None
         self.zero_based_roi: Optional[List[Tuple[int, int]]] = None
+        self.filename_to_size_map: Dict[str, int] = {}
+        for cnk in _original_chunks:
+            # since files downloaded while reading will be decompressed, we need to store the name without compression
+            filename_without_compression = cnk["filename"].replace(f".{self._compressor_name}", "")
+            self.filename_to_size_map[filename_without_compression] = cnk["chunk_bytes"]
 
     def can_delete(self, chunk_index: int) -> bool:
         if self._skip_chunk_indexes_deletion is None:
@@ -118,11 +125,18 @@ class ChunksConfig:
 
         if os.path.exists(local_chunkpath):
             self.try_decompress(local_chunkpath)
+            if self._downloader is not None:
+                # We don't want to redownload the base, but we should mark
+                # it as having been requested by something
+                self._downloader._increment_local_lock(local_chunkpath.replace(f".{self._compressor_name}", ""))
+                pass
             return
 
         if self._downloader is None:
             return
 
+        self._downloader._increment_local_lock(local_chunkpath.replace(f".{self._compressor_name}", ""))
+
         self._downloader.download_chunk_from_index(chunk_index)
 
         self.try_decompress(local_chunkpath)
@@ -226,7 +240,9 @@ class ChunksConfig:
 
         begin = self._intervals[index.chunk_index][0]
 
-        return local_chunkpath, begin, chunk["chunk_bytes"]
+        filesize_bytes = chunk["chunk_bytes"]
+
+        return local_chunkpath, begin, filesize_bytes
 
     def _get_chunk_index_from_filename(self, chunk_filename: str) -> int:
         """Retrieves the associated chunk_index for a given chunk filename."""
@@ -250,8 +266,16 @@ class ChunksConfig:
         cache_index_filepath = os.path.join(cache_dir, _INDEX_FILENAME)
 
         if isinstance(remote_dir, str):
-            downloader = get_downloader_cls(remote_dir, cache_dir, [], storage_options)
-            downloader.download_file(os.path.join(remote_dir, _INDEX_FILENAME), cache_index_filepath)
+            # for remote_dir, we try downloading `index.json` file.
+            # If the files are stored on HF, they don't have an index file, so we can skip downloading it.
+            if remote_dir.startswith("hf://"):
+                if not os.path.exists(cache_index_filepath):
+                    raise RuntimeError(
+                        f"This should not have happened. No index.json file found in cache: {cache_index_filepath}"
+                    )
+            else:
+                downloader = get_downloader(remote_dir, cache_dir, [], storage_options)
+                downloader.download_file(os.path.join(remote_dir, _INDEX_FILENAME), cache_index_filepath)
 
         if not os.path.exists(cache_index_filepath):
             return None
@@ -284,16 +308,16 @@ def load_subsampled_chunks(subsampled_files: List[str], original_chunks: List[Di
 
     assert len(_subsampled_chunks) == len(subsampled_files)
 
-    filename_dict = {}
+    filename_dict = defaultdict(list)
 
     # Populate the dictionary with filenames and their indices
     for index, filename in enumerate(subsampled_files):
-        filename_dict[filename] = index
+        filename_dict[filename].append(index)
 
     for curr_chunk in original_chunks:
         if curr_chunk["filename"] in filename_dict:
-            idx = filename_dict[curr_chunk["filename"]]
-            _subsampled_chunks[idx] = curr_chunk
+            for idx in filename_dict[curr_chunk["filename"]]:
+                _subsampled_chunks[idx] = curr_chunk
 
     # if any idx of _subsampled_chunks is None, means,
     # some elements in subsampled_files were not actually part of chunks
