@@ -11,20 +11,19 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import logging
 import os
-from logging import Logger
 from time import time
 from typing import Any, Dict, List, Optional, Tuple, Union
 
 import numpy as np
 from torch.utils.data import IterableDataset
 
-from litdata.constants import (
-    _INDEX_FILENAME,
-)
+from litdata import __version__
+from litdata.constants import _INDEX_FILENAME
+from litdata.helpers import _check_version_and_prompt_upgrade
 from litdata.streaming import Cache
-from litdata.streaming.downloader import get_downloader_cls  # noqa: F401
-from litdata.streaming.item_loader import BaseItemLoader
+from litdata.streaming.item_loader import BaseItemLoader, ParquetLoader
 from litdata.streaming.resolver import Dir, _resolve_dir
 from litdata.streaming.sampler import ChunkedIndex
 from litdata.streaming.serializers import Serializer
@@ -32,12 +31,14 @@ from litdata.streaming.shuffle import FullShuffle, NoShuffle, Shuffle
 from litdata.utilities.dataset_utilities import _should_replace_path, _try_create_cache_dir, subsample_streaming_dataset
 from litdata.utilities.encryption import Encryption
 from litdata.utilities.env import _DistributedEnv, _is_in_dataloader_worker, _WorkerEnv
+from litdata.utilities.format import _convert_bytes_to_int
+from litdata.utilities.hf_dataset import index_hf_dataset
 from litdata.utilities.shuffle import (
     _find_chunks_per_workers_on_which_to_skip_deletion,
     _map_node_worker_rank_to_chunk_indexes_to_not_delete,
 )
 
-logger = Logger(__name__)
+logger = logging.getLogger(__name__)
 
 
 class StreamingDataset(IterableDataset):
@@ -46,6 +47,7 @@ class StreamingDataset(IterableDataset):
     def __init__(
         self,
         input_dir: Union[str, "Dir"],
+        cache_dir: Optional[Union[str, "Dir"]] = None,
         item_loader: Optional[BaseItemLoader] = None,
         shuffle: bool = False,
         drop_last: Optional[bool] = None,
@@ -56,11 +58,15 @@ class StreamingDataset(IterableDataset):
         encryption: Optional[Encryption] = None,
         storage_options: Optional[Dict] = {},
         max_pre_download: int = 2,
+        index_path: Optional[str] = None,
+        force_override_state_dict: bool = False,
     ) -> None:
         """The streaming dataset can be used once your data have been optimised using the DatasetOptimiser class.
 
-        Arguments:
+        Args:
             input_dir: Path to the folder where the input data is stored.
+            cache_dir: Path to the folder where the cache data is stored. If not provided, the cache will be stored
+                in the default cache directory.
             item_loader: The logic to load an item from a chunk.
             shuffle: Whether to shuffle the data.
             drop_last: If `True`, drops the last items to ensure that
@@ -74,22 +80,46 @@ class StreamingDataset(IterableDataset):
             encryption: The encryption object to use for decrypting the data.
             storage_options: Additional connection options for accessing storage services.
             max_pre_download: Maximum number of chunks that can be pre-downloaded by the StreamingDataset.
+            index_path: Path to `index.json` for the Parquet dataset.
+                If `index_path` is a directory, the function will look for `index.json` within it.
+                If `index_path` is a full file path, it will use that directly.
+            force_override_state_dict: Boolean flag for allowing local arguments to override a loaded state dict.
 
         """
+        _check_version_and_prompt_upgrade(__version__)
+
         super().__init__()
         if not isinstance(shuffle, bool):
             raise ValueError(f"Shuffle should be a boolean. Found {shuffle}")
 
-        if not isinstance(subsample, float) or subsample <= 0.0 or subsample > 1.0:
-            raise ValueError("subsample must be a float with value between 0 and 1.")
+        if not isinstance(subsample, float) or subsample <= 0.0:
+            raise ValueError("subsample must be a float with value greater than 0.")
 
         input_dir = _resolve_dir(input_dir)
+        cache_dir = _resolve_dir(cache_dir)
+
+        if input_dir.url is not None and input_dir.url.startswith("hf://"):
+            if index_path is None:
+                # No index_path was provided. Attempt to load it from cache or generate it dynamically on the fly.
+                index_path = index_hf_dataset(input_dir.url)
+                cache_dir.path = index_path
+                input_dir.path = index_path
+
+            if item_loader is not None and not isinstance(item_loader, ParquetLoader):
+                raise ValueError(
+                    "Invalid item_loader for hf://datasets. "
+                    "The item_loader must be an instance of ParquetLoader. "
+                    "Please provide a valid ParquetLoader instance."
+                )
+
+            item_loader = item_loader or ParquetLoader()
 
         self.input_dir = input_dir
+        self.cache_dir = cache_dir
         self.subsampled_files: List[str] = []
         self.region_of_interest: List[Tuple[int, int]] = []
         self.subsampled_files, self.region_of_interest = subsample_streaming_dataset(
-            self.input_dir, item_loader, subsample, shuffle, seed, storage_options
+            self.input_dir, self.cache_dir, item_loader, subsample, shuffle, seed, storage_options, index_path
         )
 
         self.item_loader = item_loader
@@ -98,7 +128,7 @@ class StreamingDataset(IterableDataset):
 
         if self.distributed_env.world_size > 1:
             if drop_last is False:
-                logger.warn(
+                logger.warning(
                     "You're operating within a distributed environment and have disabled the `drop_last` option. "
                     "Please note that this configuration may lead to training interruptions if your system depends "
                     "on distributed collectives."
@@ -111,6 +141,17 @@ class StreamingDataset(IterableDataset):
         self.seed = seed
         self.max_cache_size = max_cache_size
 
+        max_cache_size_in_bytes = int(
+            _convert_bytes_to_int(max_cache_size) if isinstance(max_cache_size, str) else max_cache_size,
+        )
+        min_cache_size_in_bytes = _convert_bytes_to_int("25GB")
+        if max_cache_size_in_bytes < min_cache_size_in_bytes:
+            logger.warning(
+                "The provided `max_cache_size` is less than 25GB. "
+                "This may lead to performance issues during the training process. "
+                "Consider increasing the `max_cache_size` to at least 25GB to avoid potential performance degradation."
+            )
+
         self.cache: Optional[Cache] = None
         self.worker_env: Optional[_WorkerEnv] = None
         self.worker_chunks: List[int] = []
@@ -127,6 +168,7 @@ class StreamingDataset(IterableDataset):
         self.shuffler: Optional[Shuffle] = None
         self.serializers = serializers
         self._state_dict: Optional[Dict[str, Any]] = None
+        self._force_override_state_dict = force_override_state_dict
         # Has slightly different meaning in the context of the dataset
         # We consider `num_workers = 0` from `torch.utils.DataLoader` still as 1 worker (the main process)
         self.num_workers: int = 1
@@ -155,7 +197,8 @@ class StreamingDataset(IterableDataset):
     def _create_cache(self, worker_env: _WorkerEnv) -> Cache:
         if _should_replace_path(self.input_dir.path):
             cache_path = _try_create_cache_dir(
-                input_dir=self.input_dir.path if self.input_dir.path else self.input_dir.url
+                input_dir=self.input_dir.path if self.input_dir.path else self.input_dir.url,
+                cache_dir=self.cache_dir.path,
             )
             if cache_path is not None:
                 self.input_dir.path = cache_path
@@ -177,7 +220,7 @@ class StreamingDataset(IterableDataset):
         if not cache.filled:
             raise ValueError(
                 f"The provided dataset `{self.input_dir}` doesn't contain any {_INDEX_FILENAME} file."
-                " HINT: Did you successfully optimize a dataset to the provided `input_dir`?"
+                "\n HINT: Did you successfully optimize a dataset to the provided `input_dir`?"
             )
 
         return cache
@@ -399,6 +442,7 @@ class StreamingDataset(IterableDataset):
             "current_epoch": self.current_epoch,
             "input_dir_path": self.input_dir.path,
             "input_dir_url": self.input_dir.url,
+            "cache_dir_path": self.cache_dir.path,
             "item_loader": self.item_loader.state_dict() if self.item_loader else None,
             "drop_last": self.drop_last,
             "seed": self.seed,
@@ -417,63 +461,112 @@ class StreamingDataset(IterableDataset):
         self._state_dict = None
 
     def _validate_state_dict(self) -> None:
+        if self._force_override_state_dict:
+            logger.warning(
+                "Using state dict override, may lead to unexpected behavior if you're not certain what you're doing."
+            )
+
         assert self._state_dict
         assert self.worker_env
         assert self.cache
 
         state: Dict[str, Any] = self._state_dict
         if state["shuffle"] != self.shuffle:
-            raise ValueError(
-                "The provided `shuffle` state doesn't match the current one. "
-                f"Found `{self.shuffle}` instead of `{state['shuffle']}`."
+            if not self._force_override_state_dict:
+                raise ValueError(
+                    "The provided `shuffle` state doesn't match the current one. "
+                    f"Found `{self.shuffle}` instead of `{state['shuffle']}`."
+                )
+            state["shuffle"] = self.shuffle
+            logger.warning(
+                f"Overriding state shuffle {state['shuffle']} to {self.shuffle}, "
+                "this may lead to repeated or skipped datapoints within an episode."
             )
 
         if state["num_workers"] != self.worker_env.world_size:
-            raise ValueError(
-                "The provided `num_workers` state doesn't match the current one. "
-                f"Found `{self.worker_env.world_size}` instead of `{state['num_workers']}`."
+            if not self._force_override_state_dict:
+                raise ValueError(
+                    "The provided `num_workers` state doesn't match the current one. "
+                    f"Found `{self.worker_env.world_size}` instead of `{state['num_workers']}`."
+                )
+            state["num_workers"] = self.worker_env.world_size
+            logger.warning(
+                f"Overriding num workers {state['num_workers']} to {self.worker_env.world_size}. "
+                "This may lead to repeated or skipped datapoints within an episode due to different shuffles."
             )
 
         # Note: We need to check whether the path has been resolved to its associated cache.
         # In this case, validate the cache folder is the same.
         if _should_replace_path(state["input_dir_path"]):
             cache_path = _try_create_cache_dir(
-                input_dir=state["input_dir_path"] if state["input_dir_path"] else state["input_dir_url"]
+                input_dir=state["input_dir_path"] if state["input_dir_path"] else state["input_dir_url"],
+                cache_dir=state.get("cache_dir_path"),
             )
             if cache_path != self.input_dir.path:
+                if not self._force_override_state_dict:
+                    raise ValueError(
+                        "The provided `input_dir` path state doesn't match the current one. "
+                        f"Found `{self.input_dir.path}` instead of `{cache_path}`."
+                    )
+                state["input_dir_path"] = self.input_dir.path
+                logger.warning(
+                    f"Overriding state input_dir_path {state['input_dir_path']} to {self.input_dir.path}, "
+                    "this may lead to entirely different data loading."
+                )
+
+        elif state["input_dir_path"] != self.input_dir.path:
+            if not self._force_override_state_dict:
                 raise ValueError(
                     "The provided `input_dir` path state doesn't match the current one. "
-                    f"Found `{self.input_dir.path}` instead of `{cache_path}`."
+                    f"Found `{self.input_dir.path}` instead of `{state['input_dir_path']}`."
                 )
-        elif state["input_dir_path"] != self.input_dir.path:
-            raise ValueError(
-                "The provided `input_dir` path state doesn't match the current one. "
-                f"Found `{self.input_dir.path}` instead of `{state['input_dir_path']}`."
+            state["input_dir_path"] = self.input_dir.path
+            logger.warning(
+                f"Overriding state input_dir_path {state['input_dir_path']} to {self.input_dir.path}, "
+                "this may lead to entirely different data loading."
             )
 
         if state["input_dir_url"] != self.input_dir.url:
-            raise ValueError(
-                "The provided `input_dir` URL state doesn't match the current one. "
-                f"Found `{self.input_dir.url}` instead of `{state['input_dir_url']}`."
+            if not self._force_override_state_dict:
+                raise ValueError(
+                    "The provided `input_dir` URL state doesn't match the current one. "
+                    f"Found `{self.input_dir.url}` instead of `{state['input_dir_url']}`."
+                )
+            state["input_dir_url"] = self.input_dir.url
+            logger.warning(
+                f"Overriding state input_dir_url {state['input_dir_url']} to {self.input_dir.url}, "
+                "this may lead to entirely different data loading."
             )
 
         if state["seed"] != self.seed:
-            raise ValueError(
-                "The provided `seed` state doesn't match the current one. "
-                f"Found `{self.seed}` instead of `{state['seed']}`."
+            if not self._force_override_state_dict:
+                raise ValueError(
+                    "The provided `seed` state doesn't match the current one. "
+                    f"Found `{self.seed}` instead of `{state['seed']}`."
+                )
+            state["seed"] = self.seed
+            logger.warning(
+                f"Overriding state seed {state['seed']} to {self.seed}, "
+                "this may lead to repeated or skipped datapoints within an episode."
             )
 
         if self.item_loader and state["item_loader"] != self.item_loader.state_dict():
-            raise ValueError(
-                "The provided `item_loader` state doesn't match the current one. "
-                f"Found `{self.item_loader.state_dict()}` instead of `{state['item_loader']}`."
-            )
+            if not self._force_override_state_dict:
+                raise ValueError(
+                    "The provided `item_loader` state doesn't match the current one. "
+                    f"Found `{self.item_loader.state_dict()}` instead of `{state['item_loader']}`."
+                )
+            logger.warning(f"Overriding state item_loader {state['item_loader']} to {self.item_loader.state_dict()}.")
+            state["item_loader"] = self.item_loader.state_dict()
 
         if state["drop_last"] != self.drop_last:
-            raise ValueError(
-                "The provided `drop_last` state doesn't match the current one. "
-                f"Found `{self.drop_last}` instead of `{state['drop_last']}`."
-            )
+            if not self._force_override_state_dict:
+                raise ValueError(
+                    "The provided `drop_last` state doesn't match the current one. "
+                    f"Found `{self.drop_last}` instead of `{state['drop_last']}`."
+                )
+            state["drop_last"] = self.drop_last
+            logger.warning(f"Overriding state drop_last {state['drop_last']} to {self.drop_last}.")
 
         if state["num_samples_yielded"] > len(self):
             raise ValueError(
