@@ -12,16 +12,19 @@
 # limitations under the License.
 
 import contextlib
+import logging
 import os
 import warnings
-from logging import Logger
+from contextlib import suppress
 from queue import Empty, Queue
 from threading import Event, Thread
 from typing import Any, Dict, List, Optional, Tuple, Union
 
-from litdata.constants import _TORCH_GREATER_EQUAL_2_1_0
+from filelock import FileLock, Timeout
+
+from litdata.constants import _DEBUG
 from litdata.streaming.config import ChunksConfig, Interval
-from litdata.streaming.item_loader import BaseItemLoader, PyTreeLoader
+from litdata.streaming.item_loader import BaseItemLoader, ParquetLoader, PyTreeLoader, TokensLoader
 from litdata.streaming.sampler import ChunkedIndex
 from litdata.streaming.serializers import Serializer, _get_serializers
 from litdata.utilities.encryption import Encryption
@@ -29,11 +32,8 @@ from litdata.utilities.env import _DistributedEnv, _WorkerEnv
 
 warnings.filterwarnings("ignore", message=".*The given buffer is not writable.*")
 
-if _TORCH_GREATER_EQUAL_2_1_0:
-    pass
 
-
-logger = Logger(__name__)
+logger = logging.getLogger(__name__)
 
 
 _END_TOKEN = "END"  # noqa: S105
@@ -54,6 +54,7 @@ class PrepareChunksThread(Thread):
         distributed_env: _DistributedEnv,
         max_cache_size: Optional[int] = None,
         max_pre_download: int = 2,
+        rank: Optional[int] = None,
     ) -> None:
         super().__init__(daemon=True)
         self._config = config
@@ -69,6 +70,11 @@ class PrepareChunksThread(Thread):
         self._to_delete_queue: Queue = Queue()
         self._force_stop_event = Event()
 
+        # TODO: Find a real fix to this problem
+        self._force_download_queue: Queue = Queue()
+
+        self._rank = rank
+
         # Check whether a dataset slice fits on the node
         num_bytes_per_nodes = self._config.num_bytes // self._distributed_env.num_nodes
         self._delete_chunks_when_processed = num_bytes_per_nodes > max_cache_size if max_cache_size else False
@@ -84,15 +90,66 @@ class PrepareChunksThread(Thread):
         for chunk_index in chunk_indexes:
             self._to_delete_queue.put(chunk_index)
 
+    def _remaining_locks(self, chunkpath: str) -> int:
+        countpath = chunkpath + ".cnt"
+        if not os.path.exists(countpath):
+            return 0
+        with open(countpath) as count_f:
+            try:
+                return int(count_f.read().strip())
+            except Exception:
+                return 1
+
+    def _decrement_local_lock(self, chunk_index: int) -> int:
+        """Remove a count from the local lock, return the remaining count."""
+        chunk_filepath, _, _ = self._config[ChunkedIndex(index=-1, chunk_index=chunk_index)]
+
+        countpath = chunk_filepath + ".cnt"
+        with suppress(Timeout), FileLock(countpath + ".lock", timeout=3):
+            if not os.path.exists(countpath):
+                return 0
+            with open(countpath) as count_f:
+                try:
+                    curr_count = int(count_f.read().strip())
+                except Exception:
+                    curr_count = 1
+            curr_count -= 1
+            if curr_count <= 0:
+                with contextlib.suppress(FileNotFoundError, PermissionError):
+                    os.remove(countpath)
+
+                with contextlib.suppress(FileNotFoundError, PermissionError):
+                    os.remove(countpath + ".lock")
+            else:
+                with open(countpath, "w+") as count_f:
+                    count_f.write(str(curr_count))
+            return curr_count
+        return 0
+
     def _apply_delete(self, chunk_index: int) -> None:
         """Inform the item loader of the chunk to delete."""
-        if self._config.can_delete(chunk_index):
-            chunk_filepath, _, _ = self._config[ChunkedIndex(index=-1, chunk_index=chunk_index)]
-            self._item_loader.delete(chunk_index, chunk_filepath)
+        # TODO: Fix the can_delete method
+        can_delete_chunk = self._config.can_delete(chunk_index)
+        chunk_filepath, _, _ = self._config[ChunkedIndex(index=-1, chunk_index=chunk_index)]
 
-            locak_chunk_path = chunk_filepath + ".lock"
-            if os.path.exists(locak_chunk_path):
-                os.remove(locak_chunk_path)
+        remaining_locks = self._remaining_locks(chunk_filepath)
+        if remaining_locks > 0:  # Can't delete this, something has it
+            if _DEBUG:
+                print(f"Skip delete {chunk_filepath} by {self._rank or 0}, current lock count: {remaining_locks}")
+            return
+
+        self._item_loader.delete(chunk_index, chunk_filepath)
+
+        if _DEBUG:
+            print(f"Deleted {chunk_filepath} by {self._rank or 0}. Debug: {can_delete_chunk}")
+
+        for lock_extension in [".lock", ".cnt.lock"]:
+            try:
+                locak_chunk_path = chunk_filepath + lock_extension
+                if os.path.exists(locak_chunk_path):
+                    os.remove(locak_chunk_path)
+            except FileNotFoundError:
+                pass
 
     def stop(self) -> None:
         """Receive the list of the chunk indices to download for the current epoch."""
@@ -125,17 +182,40 @@ class PrepareChunksThread(Thread):
     def _can_delete_chunk(self) -> bool:
         if self._delete_chunks_when_processed:
             return self._pre_download_counter >= self._max_pre_download - 1
-        return self._max_cache_size is not None and _get_folder_size(self._parent_cache_dir) >= self._max_cache_size
+        return (
+            self._max_cache_size is not None
+            and _get_folder_size(self._config._cache_dir, self._config) >= self._max_cache_size
+        )
 
     def _pre_load_chunk(self, chunk_index: int) -> None:
         chunk_filepath, _, _ = self._config[ChunkedIndex(index=-1, chunk_index=chunk_index)]
         self._item_loader.pre_load_chunk(chunk_index, chunk_filepath)
 
+    def _force_download(self) -> None:
+        chunk_index = _get_from_queue(self._force_download_queue)
+        if chunk_index is not None:
+            if _DEBUG:
+                chunk_filepath, _, _ = self._config[ChunkedIndex(index=-1, chunk_index=chunk_index)]
+                print(f"Requested force download for {chunk_filepath} by {self._rank}")
+
+            self._config.download_chunk_from_index(chunk_index)
+
+            # Preload item if possible to gain some time but only
+            # if this is one of the pre-downloaded chunk
+            if self._pre_download_counter > 0:
+                self._pre_load_chunk(chunk_index)
+
+            # Avoid downloading too many chunks in advance at the risk of over using the disk space
+            self._pre_download_counter += 1
+
     def run(self) -> None:
         while True:
             if self._force_stop_event.is_set():
                 self._has_exited = True
                 return
+
+            self._force_download()
+
             if self._pre_download_counter < self._max_pre_download:
                 chunk_index = _get_from_queue(self._to_download_queue)
                 if chunk_index == _END_TOKEN:
@@ -157,6 +237,11 @@ class PrepareChunksThread(Thread):
                 self._maybe_delete_chunks()
 
 
+# The BinaryReader operates as the inverse of the data optimization process:
+# 1. Loads raw bytes from chunks based on specific indices
+# 2. Uses deserializers to convert bytes back into Python objects
+# 3. Reconstructs the original data structure with the data_spec from index.json and using `tree_unflatten function`
+# 4. Supports features like compression, encryption, and distributed reading
 class BinaryReader:
     def __init__(
         self,
@@ -174,7 +259,7 @@ class BinaryReader:
     ) -> None:
         """The BinaryReader enables to read chunked dataset in an efficient way.
 
-        Arguments:
+        Args:
             cache_dir: The path to cache folder.
             subsampled_files: List of subsampled chunk files loaded from `input_dir/index.json` file.
             region_of_interest: List of tuples of {start,end} of region of interest for each chunk.
@@ -267,8 +352,15 @@ class BinaryReader:
             # Create and start the prepare chunks thread
             if self._prepare_thread is None and self._config:
                 self._prepare_thread = PrepareChunksThread(
-                    self._config, self._item_loader, self._distributed_env, self._max_cache_size, self._max_pre_download
+                    self._config,
+                    self._item_loader,
+                    self._distributed_env,
+                    self._max_cache_size,
+                    self._max_pre_download,
+                    self._rank,
                 )
+                # Attach the force download queue
+                self._item_loader._force_download_queue = self._prepare_thread._force_download_queue  # type: ignore
                 self._prepare_thread.start()
                 if index.chunk_indexes:
                     self._prepare_thread.download(index.chunk_indexes)
@@ -282,15 +374,15 @@ class BinaryReader:
                 self._last_chunk_index = index.chunk_index
 
         # Fetch the element
-        chunk_filepath, begin, chunk_bytes = self.config[index]
+        chunk_filepath, begin, filesize_bytes = self.config[index]
 
         if isinstance(self._item_loader, PyTreeLoader):
             item = self._item_loader.load_item_from_chunk(
-                index.index, index.chunk_index, chunk_filepath, begin, chunk_bytes, self._encryption
+                index.index, index.chunk_index, chunk_filepath, begin, filesize_bytes, self._encryption
             )
         else:
             item = self._item_loader.load_item_from_chunk(
-                index.index, index.chunk_index, chunk_filepath, begin, chunk_bytes
+                index.index, index.chunk_index, chunk_filepath, begin, filesize_bytes
             )
 
         # We need to request deletion after the latest element has been loaded.
@@ -304,15 +396,23 @@ class BinaryReader:
             assert self._last_chunk_index is not None
 
             # inform the chunk has been completely consumed
+            self._prepare_thread._decrement_local_lock(self._last_chunk_index)
             self._prepare_thread.delete([self._last_chunk_index])
 
+        if index.chunk_index != self._last_chunk_index:
+            # Close the memory-mapped file for the last chunk index
+            if isinstance(self._item_loader, (TokensLoader, ParquetLoader)) and self._last_chunk_index is not None:
+                self._item_loader.close(self._last_chunk_index)
+
             # track the new chunk index as the latest one
             self._last_chunk_index = index.chunk_index
 
         if index.is_last_index and self._prepare_thread:
             # inform the thread it is time to stop
+            self._prepare_thread._decrement_local_lock(index.chunk_index)
             self._prepare_thread.stop()
             self._prepare_thread = None
+            self._item_loader.close(self._last_chunk_index)
 
         return item
 
@@ -341,17 +441,51 @@ class BinaryReader:
             self._prepare_thread = None
 
 
-def _get_folder_size(path: str) -> int:
-    """Collect the size of each files within a folder.
+def _get_folder_size(path: str, config: ChunksConfig) -> int:
+    """Calculate the total size of files in a directory based on specific rules.
 
-    This method is robust to file deletion races
+    This method is robust to file deletion races.
+
+    Args:
+        path (str): Directory path to scan.
+        config (ChunksConfig): Configuration object containing filename_to_size_map.
+
+    Returns:
+        int: Total size of valid files in bytes.
 
     """
     size = 0
-    for dirpath, _, filenames in os.walk(str(path)):
-        for filename in filenames:
-            with contextlib.suppress(FileNotFoundError):
-                size += os.stat(os.path.join(dirpath, filename)).st_size
+    ignored_extensions = (".cnt", ".lock", ".json", ".zstd.bin")
+
+    # os.scan_dir is more efficient than os.listdir
+    with os.scandir(path) as dir_entries:
+        for entry in dir_entries:
+            # skip directories and symlinks
+            if not entry.is_file(follow_symlinks=False):
+                continue
+
+            filename = entry.name
+
+            # use size from config if available
+            if filename in config.filename_to_size_map:
+                size += config.filename_to_size_map[filename]
+
+            # silently ignore specified extensions
+            elif filename.endswith(ignored_extensions):
+                continue
+
+            # handle temporary files containing '.bin'
+            elif ".bin" in filename:
+                with contextlib.suppress(FileNotFoundError):
+                    size += entry.stat(follow_symlinks=False).st_size
+
+            # warn about unrecognized files
+            else:
+                logger.warning(
+                    f"Ignoring '{filename}': "
+                    "This file doesn't appear to be a valid chunk file and has been excluded from the size calculation."
+                )
+
     return size
 
 
