@@ -20,13 +20,11 @@ from contextlib import suppress
 from dataclasses import dataclass
 from pathlib import Path
 from time import sleep
-from typing import TYPE_CHECKING, Any, Literal, Optional, Union
+from typing import TYPE_CHECKING, Any, Dict, Literal, Optional, Union
 from urllib import parse
 
-import boto3
-import botocore
-
-from litdata.constants import _LIGHTNING_SDK_AVAILABLE
+from litdata.constants import _LIGHTNING_SDK_AVAILABLE, _SUPPORTED_PROVIDERS
+from litdata.streaming.fs_provider import _get_fs_provider, not_supported_provider
 
 if TYPE_CHECKING:
     from lightning_sdk import Machine
@@ -52,7 +50,7 @@ def _resolve_dir(dir_path: Optional[Union[str, Dir]]) -> Dir:
 
     assert isinstance(dir_path, str)
 
-    cloud_prefixes = ("s3://", "gs://", "azure://")
+    cloud_prefixes = ("s3://", "gs://", "azure://", "hf://")
     if dir_path.startswith(cloud_prefixes):
         return Dir(path=None, url=dir_path)
 
@@ -75,6 +73,9 @@ def _resolve_dir(dir_path: Optional[Union[str, Dir]]) -> Dir:
     if dir_path_absolute.startswith("/teamspace/s3_connections") and len(dir_path_absolute.split("/")) > 3:
         return _resolve_s3_connections(dir_path_absolute)
 
+    if dir_path_absolute.startswith("/teamspace/s3_folders") and len(dir_path_absolute.split("/")) > 3:
+        return _resolve_s3_folders(dir_path_absolute)
+
     if dir_path_absolute.startswith("/teamspace/datasets") and len(dir_path_absolute.split("/")) > 3:
         return _resolve_datasets(dir_path_absolute)
 
@@ -88,18 +89,15 @@ def _match_studio(target_id: Optional[str], target_name: Optional[str], cloudspa
     if target_id is not None and cloudspace.id == target_id:
         return True
 
-    if (
+    return bool(
         cloudspace.display_name is not None
         and target_name is not None
         and cloudspace.display_name.lower() == target_name.lower()
-    ):
-        return True
-
-    return False
+    )
 
 
 def _resolve_studio(dir_path: str, target_name: Optional[str], target_id: Optional[str]) -> Dir:
-    from lightning_cloud.rest_client import LightningClient
+    from lightning_sdk.lightning_cloud.rest_client import LightningClient
 
     client = LightningClient(max_tries=2)
 
@@ -108,10 +106,10 @@ def _resolve_studio(dir_path: str, target_name: Optional[str], target_id: Option
     project_id = os.getenv("LIGHTNING_CLOUD_PROJECT_ID", None)
 
     if cluster_id is None:
-        raise RuntimeError("The `cluster_id` couldn't be found from the environement variables.")
+        raise RuntimeError("The `LIGHTNING_CLUSTER_ID` couldn't be found from the environment variables.")
 
     if project_id is None:
-        raise RuntimeError("The `project_id` couldn't be found from the environement variables.")
+        raise RuntimeError("The `LIGHTNING_CLOUD_PROJECT_ID` couldn't be found from the environment variables.")
 
     clusters = client.cluster_service_list_project_clusters(project_id).clusters
 
@@ -140,14 +138,14 @@ def _resolve_studio(dir_path: str, target_name: Optional[str], target_id: Option
 
 
 def _resolve_s3_connections(dir_path: str) -> Dir:
-    from lightning_cloud.rest_client import LightningClient
+    from lightning_sdk.lightning_cloud.rest_client import LightningClient
 
     client = LightningClient(max_tries=2)
 
     # Get the ids from env variables
     project_id = os.getenv("LIGHTNING_CLOUD_PROJECT_ID", None)
     if project_id is None:
-        raise RuntimeError("The `project_id` couldn't be found from the environement variables.")
+        raise RuntimeError("The `LIGHTNING_CLOUD_PROJECT_ID` couldn't be found from the environment variables.")
 
     target_name = dir_path.split("/")[3]
 
@@ -161,24 +159,46 @@ def _resolve_s3_connections(dir_path: str) -> Dir:
     return Dir(path=dir_path, url=os.path.join(data_connection[0].aws.source, *dir_path.split("/")[4:]))
 
 
+def _resolve_s3_folders(dir_path: str) -> Dir:
+    from lightning_sdk.lightning_cloud.rest_client import LightningClient
+
+    client = LightningClient(max_tries=2)
+
+    # Get the ids from env variables
+    project_id = os.getenv("LIGHTNING_CLOUD_PROJECT_ID", None)
+    if project_id is None:
+        raise RuntimeError("The `LIGHTNING_CLOUD_PROJECT_ID` couldn't be found from the environment variables.")
+
+    target_name = dir_path.split("/")[3]
+
+    data_connections = client.data_connection_service_list_data_connections(project_id).data_connections
+
+    data_connection = [dc for dc in data_connections if dc.name == target_name]
+
+    if not data_connection:
+        raise ValueError(f"We didn't find any matching data connection with the provided name `{target_name}`.")
+
+    return Dir(path=dir_path, url=os.path.join(data_connection[0].s3_folder.source, *dir_path.split("/")[4:]))
+
+
 def _resolve_datasets(dir_path: str) -> Dir:
-    from lightning_cloud.rest_client import LightningClient
+    from lightning_sdk.lightning_cloud.rest_client import LightningClient
 
     client = LightningClient(max_tries=2)
 
     # Get the ids from env variables
     cluster_id = os.getenv("LIGHTNING_CLUSTER_ID", None)
     project_id = os.getenv("LIGHTNING_CLOUD_PROJECT_ID", None)
-    cloud_space_id = os.getenv("LIGHTNING_CLOUD_SPACE_ID", None)
+    studio_id = os.getenv("LIGHTNING_CLOUD_SPACE_ID", None)
 
     if cluster_id is None:
-        raise RuntimeError("The `cluster_id` couldn't be found from the environement variables.")
+        raise RuntimeError("The `LIGHTNING_CLUSTER_ID` couldn't be found from the environment variables.")
 
     if project_id is None:
-        raise RuntimeError("The `project_id` couldn't be found from the environement variables.")
+        raise RuntimeError("The `LIGHTNING_CLOUD_PROJECT_ID` couldn't be found from the environment variables.")
 
-    if cloud_space_id is None:
-        raise RuntimeError("The `cloud_space_id` couldn't be found from the environement variables.")
+    if studio_id is None:
+        raise RuntimeError("The `LIGHTNING_CLOUD_SPACE_ID` couldn't be found from the environment variables.")
 
     clusters = client.cluster_service_list_project_clusters(project_id).clusters
 
@@ -187,17 +207,17 @@ def _resolve_datasets(dir_path: str) -> Dir:
         for cloudspace in client.cloud_space_service_list_cloud_spaces(
             project_id=project_id, cluster_id=cluster_id
         ).cloudspaces
-        if cloudspace.id == cloud_space_id
+        if cloudspace.id == studio_id
     ]
 
     if not target_cloud_space:
-        raise ValueError(f"We didn't find any matching Studio for the provided id `{cloud_space_id}`.")
+        raise ValueError(f"We didn't find any matching Studio for the provided id `{studio_id}`.")
 
     target_cluster = [cluster for cluster in clusters if cluster.id == target_cloud_space[0].cluster_id]
 
     if not target_cluster:
         raise ValueError(
-            f"We didn't find a matching cluster associated with the id {target_cloud_space[0].cluster_id}."
+            f"We didn't find a matching cluster associated with the id `{target_cloud_space[0].cluster_id}`."
         )
 
     return Dir(
@@ -209,37 +229,38 @@ def _resolve_datasets(dir_path: str) -> Dir:
     )
 
 
-def _assert_dir_is_empty(output_dir: Dir, append: bool = False, overwrite: bool = False) -> None:
+def _assert_dir_is_empty(
+    output_dir: Dir, append: bool = False, overwrite: bool = False, storage_options: Dict[str, Any] = {}
+) -> None:
     if not isinstance(output_dir, Dir):
-        raise ValueError("The provided output_dir isn't a Dir Object.")
+        raise ValueError("The provided output_dir isn't a `Dir` Object.")
 
     if output_dir.url is None:
         return
 
     obj = parse.urlparse(output_dir.url)
 
-    if obj.scheme != "s3":
-        raise ValueError(f"The provided folder should start with s3://. Found {output_dir.url}.")
+    if obj.scheme not in _SUPPORTED_PROVIDERS:
+        not_supported_provider(output_dir.url)
 
-    s3 = boto3.client("s3")
+    fs_provider = _get_fs_provider(output_dir.url, storage_options)
 
-    objects = s3.list_objects_v2(
-        Bucket=obj.netloc,
-        Delimiter="/",
-        Prefix=obj.path.lstrip("/").rstrip("/") + "/",
-    )
+    is_empty = fs_provider.is_empty(output_dir.url)
 
-    # We aren't alloweing to add more data
+    # We aren't allowing to add more data
     # TODO: Add support for `append` and `overwrite`.
-    if objects["KeyCount"] > 0:
+    if not is_empty:
         raise RuntimeError(
             f"The provided output_dir `{output_dir.path}` already contains data and datasets are meant to be immutable."
-            " HINT: Did you consider changing the `output_dir` with your own versioning as a suffix?"
+            "\n HINT: Did you consider changing the `output_dir` with your own versioning as a suffix?"
         )
 
 
 def _assert_dir_has_index_file(
-    output_dir: Dir, mode: Optional[Literal["append", "overwrite"]] = None, use_checkpoint: bool = False
+    output_dir: Dir,
+    mode: Optional[Literal["append", "overwrite"]] = None,
+    use_checkpoint: bool = False,
+    storage_options: Dict[str, Any] = {},
 ) -> None:
     if mode is not None and mode not in ["append", "overwrite"]:
         raise ValueError(f"The provided `mode` should be either `append` or `overwrite`. Found {mode}.")
@@ -261,8 +282,8 @@ def _assert_dir_has_index_file(
             if os.path.exists(index_file) and mode is None:
                 raise RuntimeError(
                     f"The provided output_dir `{output_dir.path}` already contains an optimized immutable datasets."
-                    " HINT: Did you consider changing the `output_dir` with your own versioning as a suffix?"
-                    " HINT: If you want to append/overwrite to the existing dataset, use `mode='append | overwrite'`."
+                    "\n HINT: Did you consider changing the `output_dir` with your own versioning as a suffix?"
+                    "\n HINT: If you want to append/overwrite to the existing dataset, use `mode='append | overwrite'`."
                 )
 
             # delete index.json file and chunks
@@ -283,44 +304,36 @@ def _assert_dir_has_index_file(
 
     obj = parse.urlparse(output_dir.url)
 
-    if obj.scheme != "s3":
-        raise ValueError(f"The provided folder should start with s3://. Found {output_dir.url}.")
+    if obj.scheme not in _SUPPORTED_PROVIDERS:
+        not_supported_provider(output_dir.url)
 
-    s3 = boto3.client("s3")
+    fs_provider = _get_fs_provider(output_dir.url, storage_options)
 
-    prefix = obj.path.lstrip("/").rstrip("/") + "/"
+    prefix = output_dir.url.rstrip("/") + "/"
 
-    objects = s3.list_objects_v2(
-        Bucket=obj.netloc,
-        Delimiter="/",
-        Prefix=prefix,
-    )
+    is_empty = fs_provider.is_empty(output_dir.url)
 
     # No files are found in this folder
-    if objects["KeyCount"] == 0:
+    if is_empty:
         return
 
     # Check the index file exists
     try:
-        s3.head_object(Bucket=obj.netloc, Key=os.path.join(prefix, "index.json"))
+        fs_provider.exists(os.path.join(prefix, "index.json"))
         has_index_file = True
-    except botocore.exceptions.ClientError:
+    except Exception:
         has_index_file = False
 
     if has_index_file and mode is None:
         raise RuntimeError(
             f"The provided output_dir `{output_dir.path}` already contains an optimized immutable datasets."
-            " HINT: Did you consider changing the `output_dir` with your own versioning as a suffix?"
-            " HINT: If you want to append/overwrite to the existing dataset, use `mode='append | overwrite'`."
+            "\n HINT: Did you consider changing the `output_dir` with your own versioning as a suffix?"
+            "\n HINT: If you want to append/overwrite to the existing dataset, use `mode='append | overwrite'`."
         )
 
-    # Delete all the files (including the index file in overwrite mode)
-    bucket_name = obj.netloc
-    s3 = boto3.resource("s3")
-
+    # delete all the files (including the index file in overwrite mode)
     if mode == "overwrite" or (mode is None and not use_checkpoint):
-        for obj in s3.Bucket(bucket_name).objects.filter(Prefix=prefix):
-            s3.Object(bucket_name, obj.key).delete()
+        fs_provider.delete_file_or_directory(output_dir.url)
 
 
 def _get_lightning_cloud_url() -> str:
@@ -344,11 +357,11 @@ def _resolve_time_template(path: str) -> str:
 def _execute(
     name: str,
     num_nodes: int,
-    machine: Optional["Machine"] = None,
+    machine: Optional[Union["Machine", str]] = None,
     command: Optional[str] = None,
+    interruptible: bool = False,
 ) -> None:
     """Remotely execute the current operator."""
-
     if not _LIGHTNING_SDK_AVAILABLE:
         raise ModuleNotFoundError("The `lightning_sdk` is required.")
 
@@ -369,8 +382,9 @@ def _execute(
         num_instances=num_nodes,
         studio_id=studio._studio.id,
         teamspace_id=studio._teamspace.id,
-        cluster_id=studio._studio.cluster_id,
+        cloud_account=studio._studio.cluster_id,
         machine=machine or studio._studio_api.get_machine(studio._studio.id, studio._teamspace.id),
+        interruptible=interruptible,
     )
 
     has_printed = False
