@@ -17,10 +17,12 @@ import pickle
 import tempfile
 from abc import ABC, abstractmethod
 from collections import OrderedDict
+from contextlib import suppress
 from copy import deepcopy
 from typing import TYPE_CHECKING, Any, Dict, Optional, Tuple, Union
 
 import numpy as np
+import tifffile
 import torch
 from lightning_utilities.core.imports import RequirementCache
 
@@ -28,7 +30,6 @@ from litdata.constants import _NUMPY_DTYPES_MAPPING, _TORCH_DTYPES_MAPPING
 
 if TYPE_CHECKING:
     from PIL.JpegImagePlugin import JpegImageFile
-
 _PIL_AVAILABLE = RequirementCache("PIL")
 _TORCH_VISION_AVAILABLE = RequirementCache("torchvision")
 _AV_AVAILABLE = RequirementCache("av")
@@ -69,6 +70,8 @@ class PILSerializer(Serializer):
 
     @classmethod
     def deserialize(cls, data: bytes) -> Any:
+        if not _PIL_AVAILABLE:
+            raise ModuleNotFoundError("PIL is required. Run `pip install pillow`")
         from PIL import Image
 
         idx = 3 * 4
@@ -93,6 +96,9 @@ class JPEGSerializer(Serializer):
     """The JPEGSerializer serialize and deserialize JPEG image to and from bytes."""
 
     def serialize(self, item: Any) -> Tuple[bytes, Optional[str]]:
+        if not _PIL_AVAILABLE:
+            raise ModuleNotFoundError("PIL is required. Run `pip install pillow`")
+
         from PIL import Image
         from PIL.GifImagePlugin import GifImageFile
         from PIL.JpegImagePlugin import JpegImageFile
@@ -102,7 +108,8 @@ class JPEGSerializer(Serializer):
         if isinstance(item, JpegImageFile):
             if not hasattr(item, "filename"):
                 raise ValueError(
-                    "The JPEG Image's filename isn't defined. HINT: Open the image in your Dataset __getitem__ method."
+                    "The JPEG Image's filename isn't defined."
+                    "\n HINT: Open the image in your Dataset `__getitem__` method."
                 )
             if item.filename and os.path.isfile(item.filename):
                 # read the content of the file directly
@@ -120,7 +127,7 @@ class JPEGSerializer(Serializer):
             buff.seek(0)
             return buff.read(), None
 
-        raise TypeError(f"The provided item should be of type {JpegImageFile}. Found {item}.")
+        raise TypeError(f"The provided item should be of type `JpegImageFile`. Found {item}.")
 
     def deserialize(self, data: bytes) -> Union["JpegImageFile", torch.Tensor]:
         if _TORCH_VISION_AVAILABLE:
@@ -128,11 +135,9 @@ class JPEGSerializer(Serializer):
             from torchvision.transforms.functional import pil_to_tensor
 
             array = torch.frombuffer(data, dtype=torch.uint8)
-            try:
+            # Note: Some datasets like Imagenet contains some PNG images with JPEG extension, so we fallback to PIL
+            with suppress(RuntimeError):
                 return decode_jpeg(array)
-            except RuntimeError:
-                # Note: Some datasets like Imagenet contains some PNG images with JPEG extension, so we fallback to PIL
-                pass
 
         img = PILSerializer.deserialize(data)
         if _TORCH_VISION_AVAILABLE:
@@ -184,7 +189,7 @@ class TensorSerializer(Serializer):
         shape = []
         for shape_idx in range(shape_size):
             shape.append(np.frombuffer(data[8 + 4 * shape_idx : 8 + 4 * (shape_idx + 1)], np.uint32).item())
-        idx_start = 8 + 4 * (shape_idx + 1)
+        idx_start = 8 + 4 * shape_size
         idx_end = len(data)
         if idx_end > idx_start:
             tensor = torch.frombuffer(data[idx_start:idx_end], dtype=dtype)
@@ -197,7 +202,7 @@ class TensorSerializer(Serializer):
         return torch.reshape(tensor, shape)
 
     def can_serialize(self, item: torch.Tensor) -> bool:
-        return isinstance(item, torch.Tensor) and type(item) == torch.Tensor and len(item.shape) > 1
+        return isinstance(item, torch.Tensor) and len(item.shape) != 1
 
 
 class NoHeaderTensorSerializer(Serializer):
@@ -220,7 +225,7 @@ class NoHeaderTensorSerializer(Serializer):
         return torch.frombuffer(data, dtype=self._dtype) if len(data) > 0 else torch.empty((0,), dtype=self._dtype)
 
     def can_serialize(self, item: torch.Tensor) -> bool:
-        return isinstance(item, torch.Tensor) and type(item) == torch.Tensor and len(item.shape) == 1
+        return isinstance(item, torch.Tensor) and len(item.shape) == 1
 
 
 class NumpySerializer(Serializer):
@@ -250,13 +255,13 @@ class NumpySerializer(Serializer):
             shape.append(np.frombuffer(data[8 + 4 * shape_idx : 8 + 4 * (shape_idx + 1)], np.uint32).item())
 
         # deserialize the numpy array bytes
-        tensor = np.frombuffer(data[8 + 4 * (shape_idx + 1) : len(data)], dtype=dtype)
+        tensor = np.frombuffer(data[8 + 4 * shape_size : len(data)], dtype=dtype)
         if tensor.shape == shape:
             return tensor
         return np.reshape(tensor, shape)
 
     def can_serialize(self, item: np.ndarray) -> bool:
-        return isinstance(item, np.ndarray) and type(item) == np.ndarray and len(item.shape) > 1
+        return isinstance(item, np.ndarray) and len(item.shape) > 1
 
 
 class NoHeaderNumpySerializer(Serializer):
@@ -279,7 +284,7 @@ class NoHeaderNumpySerializer(Serializer):
         return np.frombuffer(data, dtype=self._dtype)
 
     def can_serialize(self, item: np.ndarray) -> bool:
-        return isinstance(item, np.ndarray) and type(item) == np.ndarray and len(item.shape) == 1
+        return isinstance(item, np.ndarray) and len(item.shape) == 1
 
 
 class PickleSerializer(Serializer):
@@ -383,13 +388,71 @@ class FloatSerializer(NumericSerializer, Serializer):
         return isinstance(data, float)
 
 
+class BooleanSerializer(Serializer):
+    """The BooleanSerializer serializes and deserializes boolean values to and from bytes."""
+
+    def serialize(self, item: bool) -> Tuple[bytes, Optional[str]]:
+        """Serialize a boolean value to bytes.
+
+        Args:
+            item: Boolean value to serialize
+
+        Returns:
+            Tuple containing the serialized bytes and None for the format string
+        """
+        return np.bool_(item).tobytes(), None
+
+    def deserialize(self, data: bytes) -> bool:
+        """Deserialize bytes back into a boolean value.
+
+        Args:
+            data: Bytes to deserialize
+
+        Returns:
+            The deserialized boolean value
+        """
+        return bool(np.frombuffer(data, dtype=np.bool_)[0])
+
+    def can_serialize(self, item: Any) -> bool:
+        """Check if the item can be serialized by this serializer.
+
+        Args:
+            item: Item to check
+
+        Returns:
+            True if the item is a boolean, False otherwise
+        """
+        return isinstance(item, bool)
+
+
+class TIFFSerializer(Serializer):
+    """Serializer for TIFF files using tifffile."""
+
+    def serialize(self, item: Any) -> Tuple[bytes, Optional[str]]:
+        if not isinstance(item, str) or not os.path.isfile(item):
+            raise ValueError(f"The item to serialize must be a valid file path. Received: {item}")
+
+        # Read the TIFF file as bytes
+        with open(item, "rb") as f:
+            data = f.read()
+
+        return data, None
+
+    def deserialize(self, data: bytes) -> Any:
+        return tifffile.imread(io.BytesIO(data))  # This is a NumPy array
+
+    def can_serialize(self, item: Any) -> bool:
+        return isinstance(item, str) and os.path.isfile(item) and item.lower().endswith((".tif", ".tiff"))
+
+
 _SERIALIZERS = OrderedDict(
     **{
         "str": StringSerializer(),
+        "bool": BooleanSerializer(),
         "int": IntegerSerializer(),
         "float": FloatSerializer(),
         "video": VideoSerializer(),
-        "tif": FileSerializer(),
+        "tifffile": TIFFSerializer(),
         "file": FileSerializer(),
         "pil": PILSerializer(),
         "jpeg": JPEGSerializer(),
