@@ -59,7 +59,8 @@ class Shuffle(ABC):
 
 class NoShuffle(Shuffle):
     """NoShuffle doesn't shuffle the items and ensure all the processes receive the same number of items if drop_last
-    is True."""
+    is True.
+    """
 
     @lru_cache(maxsize=10)
     def get_chunks_and_intervals_per_workers(
@@ -83,13 +84,10 @@ class FullShuffle(Shuffle):
     """FullShuffle shuffles the chunks and associates them to the ranks.
 
     As the number of items in a chunk varies, it is possible for a rank to end up with more or less items.
-
     To ensure the same fixed dataset length for all ranks while dropping as few items as possible,
-
     we adopt the following strategy.
 
     We compute the maximum number of items per rank (M) and iterate through the chunks and ranks
-
     until we have associated at least M items per rank.
 
     As a result, we lose at most (number of ranks) items. However, as some chunks are shared across ranks. This leads to
