@@ -21,15 +21,16 @@ from typing import Any, Dict, List, Optional, Tuple, Union
 
 import numpy as np
 
-from litdata.constants import _INDEX_FILENAME
+from litdata.constants import _INDEX_FILENAME, _POLARS_AVAILABLE, _TQDM_AVAILABLE
 from litdata.processing.utilities import get_worker_rank
 from litdata.streaming.compression import _COMPRESSORS, Compressor
-from litdata.streaming.item_loader import BaseItemLoader, PyTreeLoader
+from litdata.streaming.item_loader import BaseItemLoader, ParquetLoader, PyTreeLoader
 from litdata.streaming.serializers import Serializer, _get_serializers
 from litdata.utilities._pytree import PyTree, tree_flatten, treespec_dumps
 from litdata.utilities.encryption import Encryption, EncryptionLevel
 from litdata.utilities.env import _DistributedEnv, _WorkerEnv
 from litdata.utilities.format import _convert_bytes_to_int, _human_readable_bytes
+from litdata.utilities.parquet import get_parquet_indexer_cls
 
 
 @dataclass
@@ -58,14 +59,16 @@ class BinaryWriter:
     ):
         """The BinaryWriter enables to chunk dataset into an efficient streaming format for cloud training.
 
-        Arguments:
+        Args:
             cache_dir: The path to where the chunks will be saved.
             chunk_bytes: The maximum number of bytes within a chunk.
             chunk_size: The maximum number of items within a chunk.
             compression: The compression algorithm to use.
             encryption: The encryption algorithm to use.
+            follow_tensor_dimension: Whether to follow the tensor dimension when serializing the data.
             serializers: Provide your own serializers.
             chunk_index: The index of the chunk to start from.
+            item_loader: The object responsible to generate the chunk intervals and load an item from a chunk.
 
         """
         self._cache_dir = cache_dir
@@ -91,7 +94,7 @@ class BinaryWriter:
 
         if self._compression:
             if len(_COMPRESSORS) == 0:
-                raise ValueError("No compresion algorithms are installed.")
+                raise ValueError("No compression algorithms are installed.")
 
             if self._compression not in _COMPRESSORS:
                 raise ValueError(
@@ -155,7 +158,6 @@ class BinaryWriter:
 
     def serialize(self, items: Any) -> Tuple[bytes, Optional[int]]:
         """Serialize a dictionary into its binary format."""
-
         # Flatten the items provided by the users
         flattened, data_spec = tree_flatten(items)
 
@@ -204,8 +206,34 @@ class BinaryWriter:
             sizes.append(serializer.size if hasattr(serializer, "size") else len(serialized_item))
 
     def _create_chunk(self, filename: str, on_done: bool = False) -> bytes:
-        """Create a binary chunk from all the binarized items."""
-        items = []
+        """Creates a binary chunk file from serialized items."""
+        # The chunk's binary format is structured as follows:
+
+        # +------------+---------------+-------------+
+        # | num_items  | offset_array  | item_data   |
+        # +------------+---------------+-------------+
+        # | uint32     | uint32[N+1]   | bytes       |
+        # | 4 bytes    | 4*(N+1) bytes | variable    |
+        # +------------+---------------+-------------+
+
+        # Where:
+        # - num_items: Number of items in the chunk (N)
+        # - offset_array: Array of N+1 offsets indicating where each item begins/ends
+        # - item_data: Concatenated binary data of all items
+
+        # Example:
+        #     For a chunk with 3 items of sizes [10, 20, 15] bytes:
+        #     - num_items = 3 (4 bytes)
+        #     - offset_array = [start, start+10, start+30, start+45]
+        #       where start = 4 + (4 * 4) = 20 bytes (header size)
+        #     - item_data = concatenated bytes of all items
+
+        # This format allows direct access to any item by reading its offset
+        #   from `offset_array[i]` to `offset_array[i+1]`.
+        # Then, read bytes from `offset_start` to `offset_end` to get the item bytes.
+        # Now, item_loader can use these raw bytes to deserialize the item.
+
+        items: List[Item] = []
 
         if on_done:
             indices = sorted(self._serialized_items.keys())
@@ -226,19 +254,22 @@ class BinaryWriter:
                 f" Found {self._pretty_serialized_items()} with boundaries: {self._min_index}, {self._max_index}."
             )
 
-        num_items = np.uint32(len(items))
-        sizes = list(map(len, items))
-        offsets = np.array([0] + sizes).cumsum().astype(np.uint32)
+        num_items = np.uint32(len(items))  # total number of items in the chunk
+        sizes = list(map(len, items))  # list of sizes (length of bytes) of each item
+        offsets = np.array([0] + sizes).cumsum().astype(np.uint32)  # let's say: [0, 10, 30, 45]
+
+        # add the number of bytes taken to store (num_items and offsets). Let's say 60: offsets -> [60, 70, 90, 105]
         offsets += len(num_items.tobytes()) + len(offsets.tobytes())
         sample_data = b"".join([item.data for item in items])
-        data = num_items.tobytes() + offsets.tobytes() + sample_data
 
-        current_chunk_bytes = sum([item.bytes for item in items])
+        # combine all bytes data which will be written to the chunk file
+        data = num_items.tobytes() + offsets.tobytes() + sample_data
 
         # Whether to encrypt the data at the chunk level
         if self._encryption and self._encryption.level == EncryptionLevel.CHUNK:
             data = self._encryption.encrypt(data)
-            current_chunk_bytes = len(data)
+
+        current_chunk_bytes = len(data)
 
         if self._chunk_bytes and current_chunk_bytes > self._chunk_bytes:
             warnings.warn(
@@ -289,8 +320,8 @@ class BinaryWriter:
 
     def add_item(self, index: int, items: Any) -> Optional[str]:
         """Given an index and items will serialize the items and store an Item object to the growing
-        `_serialized_items`."""
-
+        `_serialized_items`.
+        """
         if index in self._serialized_items:
             raise ValueError(f"The provided index {index} already exists in the cache.")
 
@@ -413,7 +444,8 @@ class BinaryWriter:
 
     def merge(self, num_workers: int = 1, node_rank: Optional[int] = None) -> None:
         """Once all the workers have written their own index, the merge function is responsible to read and merge them
-        into a single index."""
+        into a single index.
+        """
         num_workers = num_workers or 1
 
         # Only for non rank 0
@@ -443,7 +475,7 @@ class BinaryWriter:
         """Once all the workers have written their own index, the merge function is responsible to read and merge them
         into a single index.
 
-        Arguments:
+        Args:
             node_rank: The node rank of the index file
             existing_index: Existing index to be added to the newly created one.
 
@@ -528,3 +560,84 @@ class BinaryWriter:
             json.dump(checkPoint, f)
 
         return checkpoint_filepath
+
+
+def index_parquet_dataset(
+    pq_dir_url: str,
+    cache_dir: Optional[str] = None,
+    storage_options: Optional[Dict] = {},
+    num_workers: int = 4,
+) -> None:
+    """Index a Parquet dataset from a specified URL.
+
+
+    This function scans the metadata of all `.parquet` files in the specified directory URL, extracts metadata
+    such as file size, chunk size, and data types, and indexes them. The index is saved as a JSON file in the
+    specified cache directory.
+
+    Args:
+        pq_dir_url (str): URL of the directory containing the Parquet files.
+        cache_dir (Optional[str]): Local cache directory for storing temporary files.
+            For HF dataset, index.json file will be stored here.
+        storage_options (Optional[Dict]): Additional storage options for accessing the Parquet files.
+        num_workers (int): Number of workers to download metadata of Parquet files and index them.
+
+    Raises:
+        ModuleNotFoundError: If the required `polars` module is not installed.
+    """
+    if not _POLARS_AVAILABLE:
+        raise ModuleNotFoundError("Please, run: `pip install polars`")
+
+    pq_chunks_info = []
+    config: Dict[str, Any] = {
+        "compression": None,
+        "chunk_size": None,
+        "chunk_bytes": None,
+        "data_format": [],
+        "data_spec": None,
+        "encryption": None,
+        "item_loader": ParquetLoader.__name__,
+    }
+
+    pq_dir_class = get_parquet_indexer_cls(pq_dir_url, cache_dir, storage_options, num_workers)
+
+    if _TQDM_AVAILABLE:
+        from tqdm.auto import tqdm as _tqdm
+
+        pbar = _tqdm(
+            desc="Indexing progress",
+            total=len(pq_dir_class.files),
+            smoothing=0,
+            mininterval=1,
+            leave=True,
+            dynamic_ncols=True,
+            unit="step",
+        )
+
+    results = {}
+    # iterate through the directory and index each file ending with ".parquet"
+    for file_metadata, order in pq_dir_class:
+        chunk_dtypes = file_metadata["data_types"]
+
+        if len(config["data_format"]) != 0 and config["data_format"] != chunk_dtypes:
+            raise Exception(
+                "The config isn't consistent between chunks. This shouldn't have happened."
+                f"Found {config}; {chunk_dtypes}."
+            )
+        config["data_format"] = chunk_dtypes
+        chunk_info = {
+            "chunk_bytes": file_metadata["file_size"],
+            "chunk_size": file_metadata["num_rows"],
+            "filename": file_metadata["file_name"],
+            "dim": None,
+        }
+        results[order] = chunk_info
+        if _TQDM_AVAILABLE:
+            pbar.update(1)
+
+    for i in sorted(results.keys()):
+        pq_chunks_info.append(results[i])
+
+    del results
+    print(flush=True)  # to prevent truncated printing when using concurrent threads/processes
+    pq_dir_class.write_index(pq_chunks_info, config)
